/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * Used to enumerate over an ordered list of strings.
 */

[scriptable, uuid(50d3ef6c-9380-4f06-9fb2-95488f7d141c)]
interface nsIStringEnumerator : nsISupports
{
    boolean hasMore();
    AString getNext();
};

[scriptable, uuid(9bdf1010-3695-4907-95ed-83d0410ec307)]
interface nsIUTF8StringEnumerator : nsISupports
{
    boolean hasMore();
    AUTF8String getNext();
};

