/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[builtinclass, uuid(f1a2fd3a-c6ac-4ee2-a700-5d285d5e3fff)]
interface nsIDOMWebGLRenderingContext : nsISupports
{
  // get an underlying GL parameter, without any WebGL intervention.
  // Most useful for querying GL_VENDOR/GL_RENDERER for identifying
  // the underlying renderer to the user.
  [noscript] DOMString mozGetUnderlyingParamString(in unsigned long pname);
};
